<?php

namespace Drupal\page_to_pdf\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Defines controller for generating PDF file.
 */
class NodePdfPageController extends ControllerBase {

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Render node page to generate PDF.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node.
   *
   * @return string
   *   Return PDF node content.
   */
  public function view(NodeInterface $node) {
    $node_to_view = $node;

    // If route is 'latest', load the latest revision.
    if (\Drupal::routeMatch()->getRouteName() === 'page_to_pdf.node_pdf_revision_page_controller') {
      $node_storage = \Drupal::entityTypeManager()->getStorage('node');
      $latest_vid = $node_storage->getLatestRevisionId($node->id());
      $node_to_view = $node_storage->loadRevision($latest_vid);
    }

    $pdf_settings = $this->configFactory->get('page_to_pdf.pdf_settings');
    $content_types = $pdf_settings->get('content_types') ?? [];

    // Return page not found error if PDF generating isn't enabled for this
    // content type.
    if (empty($content_types[$node_to_view->getType()])) {
      throw new NotFoundHttpException();
    }

    $view_builder = $this->entityTypeManager->getViewBuilder('node');
    $build = [
      '#theme' => 'node_pdf_content',
      '#node_view' => $view_builder->view($node_to_view, $content_types[$node_to_view->getType()]['view_mode']),
      '#cache' => [
        'tags' => [
          'node:' . $node_to_view->id(),
        ],
      ],
    ];

    // Add PageJS library.
    if (!empty($content_types[$node_to_view->getType()]['pagejs'])) {
      $build['#attached']['library'][] = 'page_to_pdf/pagedjs';
    }

    return $build;
  }

}
