<?php

namespace Drupal\page_to_pdf\Plugin\PdfGenerator;

use Drupal\node\NodeInterface;
use Drupal\page_to_pdf\Plugin\PdfGeneratorBase;

/**
 * Provides default PDF generator.
 *
 * @PdfGenerator(
 *   id = "default",
 *   label = @Translation("Default"),
 * )
 */
class DefaultPdfGenerator extends PdfGeneratorBase {

  /**
   * {@inheritdoc}
   */
  public function batchSteps(NodeInterface $node): array {
    $operations = [
      [
        [__CLASS__, 'runPluginStep'],
        [
          $this->pluginId,
          'generateMainPdf',
          $node->id(),
          $node->language()->getId(),
        ],
      ],
      [
        [__CLASS__, 'runPluginStep'],
        [
          $this->pluginId,
          'saveFinalPdf',
          $node->id(),
          $node->language()->getId(),
          NULL,
        ],
      ],
      [
        [__CLASS__, 'runPluginStep'],
        [
          $this->pluginId,
          'generateBatchPdfLink',
          $node->id(),
          $node->language()->getId(),
        ],
      ],
    ];

    // Allowing other modules to add batch operations.
    $this->moduleHandler->alter('page_to_pdf_node_operations', $node, $operations);

    return $operations;

  }

  /**
   * {@inheritdoc}
   */
  public function generateMainPdf(int $node_id, string $language_id): void {
    $node = $this->loadNode($node_id);
    if ($node->language()->getId() !== $language_id
        && $node->hasTranslation($language_id)) {
      $node = $node->getTranslation($language_id);
    }

    $this->generatePdf(
      $this->getNodePdfPage($node),
      $this->getTemporaryFilePath($node)
    );
  }

}
